@@ -12,15 +12,13 @@ use Mail::Sendmail qw(sendmail);
 use Storable qw(store retrieve);
 use Sys::Hostname qw(hostname);
 
-our $VERSION = '0.04_02';
+our $VERSION = '0.01';
 
 sub new
 {
     my $class = shift;
     my %args = @_;
 
-    my $defined_or = sub { defined $_[0] ? $_[0] : $_[1] };
-
     my $self = bless {
         Config => {
             input_file      => $args{input_file} || '/proc/user_beancounters',
@@ -33,9 +31,9 @@ sub new
                 to      => $args{mail}->{to}      || 'root@localhost',
                 subject => $args{mail}->{subject} || 'vzwatchd: NOTICE',
             },
-            sleep   => $defined_or->($args{sleep}, 60),
-            verbose => $defined_or->($args{verbose}, false),
-            _tests  => $defined_or->($args{_tests}, false),
+            sleep   => $args{sleep}   || 60,
+            verbose => $args{verbose} || false,
+            _tests  => $args{_tests}  || false,
         }
     }, ref($class) || $class;
 
@@ -48,7 +46,9 @@ sub process
 {
     my $self = shift;
 
-    delete @$self{qw(data stored)};
+    foreach my $member (qw(data stored changed)) {
+        delete $self->{$member};
+    }
 
     $self->_get_data_running;
     $self->_get_data_file;
@@ -60,8 +60,10 @@ sub _init
 {
     my $self = shift;
 
-    eval { store({}, $self->{Config}->{data_file}) }
-      or croak "Cannot store to $self->{Config}->{data_file}: $!";
+    if (!-e $self->{Config}->{data_file}) {
+        eval { store({}, $self->{Config}->{data_file}) }
+          or croak "Cannot store to $self->{Config}->{data_file}: $!";
+    }
 
     my $pkg_tmpl = join '::', (__PACKAGE__, '_template');
     no strict 'refs';
@@ -109,33 +111,29 @@ sub _get_data_running
     }
 
     my $re = qr{
-                      \s*?
-         (?:\d+?\:)?  \s+?
-         (?:\w+?)     \s+?
-         (?:(?:\d+?)  \s*?){5}
+                      \s*
+         (?:\d+?\:)?  \s+
+         (?:\w+)      \s+
+         (?:(?:\d+)   \s*){5}
     }x;
 
-    my $uid;
-
-    my @names = grep {
-      !$self->{excluded}->{$_}
-        ? $_ : ()
-    } @{$self->{Config}->{_field_names}};
+    my ($uid, $res);
 
     local $1;
     while ($output =~ /^($re)$/gm) {
         my $line = $1;
-        if ($line =~ /^ \s+? (\d+?)\:/gx) {
+        if ($line =~ s/^\s*(\d+?)\://) {
             $uid = $1;
         }
-        my $res;
-        if ($line =~ /\G \s+? (\w+)/gx) {
+        if ($line =~ s/^\s*(\w+)\s*//) {
             $res = $1;
         }
-        my @fields;
-        if ($line =~ /\G \s+ (.*) $/x) {
-            @fields = split /\s+/, $1;
-        }
+        my @fields = split /\s+/, $line;
+        my @names = grep {
+          !$self->{excluded}->{$_}
+            ? $_ : ()
+        } @{$self->{Config}->{_field_names}};
+
         push @{$self->{data}{$uid}{$res}},
           { map { $names[$_] => $fields[$_] } (0 .. $#fields) };
     }
@@ -157,15 +155,14 @@ sub _compare_data
     {
         my ($uid, $res, $i) = @_;
 
-        my ($data, $stored) = map {
-          my $type = $_; sub { $self->{$type}{$uid}{$res}->[$i]->{$_[0]} || 0 }
-        } qw(data stored);
+        my $data   = sub { $self->{data}{$uid}{$res}->[$i]->{$_[0]}   || 0 };
+        my $stored = sub { $self->{stored}{$uid}{$res}->[$i]->{$_[0]} || 0 };
 
         return scalar grep { $data->($_) > $stored->($_) } @{$self->{Config}->{monitor_fields}};
     };
 
-    foreach my $uid (sort {$a <=> $b} keys %{$self->{stored}}) {
-        foreach my $res (sort {$a cmp $b} keys %{$self->{stored}{$uid}}) {
+    foreach my $uid (keys %{$self->{stored}}) {
+        foreach my $res (keys %{$self->{stored}{$uid}}) {
             foreach my $index (0 .. $#{$self->{stored}{$uid}{$res}}) {
                 if ($has_changed->($uid, $res, $index)) {
                     $self->_create_report($uid, $res, $index);
@@ -178,6 +175,7 @@ sub _compare_data
 sub _create_report
 {
     my $self = shift;
+
     my ($uid, $res, $index) = @_;
 
     if ($self->{Config}->{_tests}) {
@@ -204,6 +202,7 @@ sub _put_data_file
 sub _prepare_report
 {
     my $self = shift;
+
     my ($uid, $res, $index) = @_;
 
     my @fixed_fields = ($uid, $res) x 2;
@@ -237,44 +236,47 @@ sub _prepare_report
         push @values, [ @v ];
     }
 
-    my %changed;
     foreach my $field (keys %{$mapping[0]->[1]}) {
         if ($mapping[0]->[1]->{$field} != $mapping[1]->[1]->{$field}) {
-            $changed{$self->{index}->{$field}} = true;
+            $self->{changed}->{$self->{index}->{$field}} = true;
         }
     }
 
     my $tmpl = \@values;
-    my $report = $self->{template};
+    my $template = $self->{template};
 
-    local $1;
-
-    while (my ($var) = $report =~ /(\$\S+)/) {
-        unless ($report =~ /\Q$var\E$/m) {
-            my $len = length($var) - length do { eval $var };
-            $report =~ s/(?<=\Q$var\E)/' ' x $len/e;
+    while ($template =~ /(\$\S+)/) {
+        unless ($template =~ /(\Q$1\E)$/m) {
+            my $len = length($1) - length do { eval $1 };
+            my $space = ' ' x $len;
+            $template =~ s/(\Q$1\E)/$1$space/;
         }
-        $report =~ s/(\Q$var\E)/$1/ee;
+        $template =~ s/(\$\S+)/$1/ee;
     }
 
-    while (my ($pos) = $report =~ /\((\d+)\)/) {
-        my $marked = $changed{$pos} ? '*' : ' ';
-        $report =~ s/\($pos\)/  $marked/;
+    while ($template =~ /\((\d+)\)/) {
+        if ($self->{changed}->{$1}) {
+            $template =~ s/\($1\)/  \*/;
+        }
+        else {
+            $template =~ s/\($1\)/   /;
+        }
     }
 
-    return $report;
+    return $template;
 }
 
 sub _send_mail
 {
     my $self = shift;
+
     my ($report) = @_;
 
     my %mail = (
         From    => $self->{Config}->{mail}->{from},
         To      => $self->{Config}->{mail}->{to},
         Subject => $self->{Config}->{mail}->{subject},
-        Message => <<"EOT",
+        Message => <<EOT,
 ${\hostname}
 
 $report
@@ -321,8 +323,6 @@ mail notifications whenever important values change (as defined by the user).
 The recommended usage of this application module is to use the provided
 L<vzwatchd> daemon.
 
-Version 2.5 of the F<user_beancounters> file is supported.
-
 =head1 CONSTRUCTOR
 
 =head2 new
@@ -413,19 +413,12 @@ Location of the data file. Defaults to F<$HOME/vzwatchd.dat>.
 
 =item * Cannot parse anything else than the F<user_beancounters> file.
 
-=item * Only version 2.5 of the F<user_beancounters> file is known to work.
-
 =item * Fixed mail report template.
 
 =item * No more than one mail recipient.
 
 =back
 
-=head1 BUGS & CAVEATS
-
-Note that the sender and recipient mail address options with defaults
-might require adjustment in order for notifications to be delivered.
-
 =head1 SEE ALSO
 
 L<OpenVZ::BC>, L<http://openvz.org>
@@ -439,6 +432,6 @@ Steven Schubiger <schubiger@cpan.org>
 This program is free software; you may redistribute it and/or
 modify it under the same terms as Perl itself.
 
-See L<http://dev.perl.org/licenses/>
+See L<http://www.perl.com/perl/misc/Artistic.html>
 
 =cut
